(**************************************************************************
* BlockCAD, a program for building with Lego like blocks on the computer
* Copyright (C) 1998  Anders Isaksson
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
* -------------------------------------------------------------------------
* I hope You send me a copy of any important changes/additions You do.
*
* E-mail: isaksson.etuna@ebox.tninet.se
*
**************************************************************************)

unit CEJUtils;

interface

uses
  Windows,
  Dialogs,
  Classes
  ;

{-------  WinExecAndWait --------------------------------------------}

function WinExecAndWait(const aCmdLine: string;
  const aDirectory: string;
  const aCmdShow: integer): Boolean;

function WinExecNoWait(const aCmdLine: string;
  const aDirectory: string;
  const aCmdShow: integer): Boolean;

{--------------------------------------------------------------------}

function Split(s: string; t: TStringList): integer;

implementation

uses
  SysUtils
  ;

{-----------------------------------------------------------------}

const
  PATHBUFFERSIZE    = 257;

function WinExecForMulti(const aCmdLine: string;
  const aDirectory: string;
  const aCmdShow: integer): THandle;
var
  CurrentDir        : string;
  StartupInfo       : TStartupInfo;
  ProcessInformation: TProcessInformation;
  CurrentDirBuff    : PChar;
begin
  CurrentDir := aDirectory;
  if CompareText(CurrentDir, '') = 0 then begin
    CurrentDirBuff := StrAlloc(PATHBUFFERSIZE);
    try
      if (GetCurrentDirectory(PATHBUFFERSIZE, CurrentDirBuff) > 0) then
        CurrentDir := StrPas(CurrentDirBuff);
    finally
      StrDispose(CurrentDirBuff);
    end;
  end;
  GetStartupInfo(StartupInfo);
  StartupInfo.WShowWindow := aCmdShow;
  if CreateProcess(nil,                 {ApplicationName}
    PChar(aCmdLine),                    {lpCommandLine }
    nil,                                {lpProcessAttributes}
    nil,                                {lpThreadAttribute}
    False,                              {bInheritedHandles}
    NORMAL_PRIORITY_CLASS,              {dwCreationFlags}
    nil,                                {lpEnvironment}
    PChar(CurrentDir),                  {lpCurrentDirectory}
    StartupInfo,                        {lpStartupInfo}
    ProcessInformation                  {lpProcessInformation}
    ) then                              {Success}
    Result := ProcessInformation.hProcess
  else                                  {Fail} begin
    Result := 0;
    ShowMessage(IntToStr(GetLastError));
  end
end;

{-----------------------------------------------------------------}

function WinExecAndWait(const aCmdLine: string;
  const aDirectory: string;
  const aCmdShow: integer): Boolean;
var
  Handle            : THandle;
begin
  Handle := WinExecForMulti(aCmdLine, aDirectory, aCmdShow);
  if Handle > 0 then begin
    WaitForSingleObject(Handle, INFINITE);
    CloseHandle(Handle);
    Result := True;
  end
  else
    Result := False;
end;

function WinExecNoWait(const aCmdLine: string;
  const aDirectory: string;
  const aCmdShow: integer): Boolean;
var
  Handle            : THandle;
begin
  Handle := WinExecForMulti(aCmdLine, aDirectory, aCmdShow);
  if Handle > 0 then begin
    CloseHandle(Handle);
    Result := True;
  end
  else
    Result := False;
end;

{-----------------------------------------------------------------}

function Split(s: string; t: TStringList): integer;
var
  p                 : integer;
begin
  t.Clear;
  p := Pos(':', s);
  while p > 0 do begin
    t.Add(Copy(s, 1, p - 1));
    Delete(s, 1, p);
    p := Pos(':', s);
  end;
  if Length(s) > 0 then
    t.Add(s);
  Result := t.Count;
end;

{-----------------------------------------------------------------}

end.
