unit Sequencer;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, TB97Ctls, TB97Tlbr, TB97, Main, ExtCtrls, RXSpin;

type
  TfrmSeq = class(TForm)
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    SeqPanel: TPanel;
    btnClear: TToolbarButton97;
    btnBegin: TToolbarButton97;
    btnBack: TToolbarButton97;
    edFrame: TEdit97;
    btnForw: TToolbarButton97;
    btnEnd: TToolbarButton97;
    btnPlayStop: TToolbarButton97;
    btnSnap: TToolbarButton97;
    btnNoSnap: TToolbarButton97;
    btnOpen: TToolbarButton97;
    btnSave: TToolbarButton97;
    edDelay: TEdit97;
    ListBox1: TListBox;
    ToolbarButton971: TToolbarButton97;
    spFrames: TRxSpinEdit;
    procedure btnClearClick(Sender: TObject);
    procedure btnBeginClick(Sender: TObject);
    procedure btnBackClick(Sender: TObject);
    procedure btnForwClick(Sender: TObject);
    procedure btnEndClick(Sender: TObject);
    procedure btnPlayStopClick(Sender: TObject);
    procedure btnSnapClick(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnNoSnapClick(Sender: TObject);
  private
    procedure UpdateFrameEdit;
    procedure UpdateView;
    function GrabSettings: TSettings;
    { Private declarations }
  public
    { Public declarations }
    procedure Attach(Form: TForm);
    procedure Detach;
    function Attached(Form: TForm): boolean;
  end;

var
  frmSeq            : TfrmSeq;

implementation

{$R *.DFM}

uses
  LCadPc2
  ;

var
  fSeq              : array of TSettings;
  CurFrame          : integer;

function TfrmSeq.GrabSettings: TSettings;
begin
  MainForm.SetBase;
  Result.ScaleBarPos := Round(10.0 / Settings.Scale);
  Result := Settings;
  Result.RotXPos := MainForm.RotX.Position;
  Result.RotZPos := MainForm.RotZ.Position;
  Result.ScaleBarPos := MainForm.ScaleBar.Position;
  Result.OffsetXPos := MainForm.OffsetX.Position;
  Result.OffsetYPos := MainForm.OffsetY.Position;
  Result.Frames := spFrames.AsInteger;
  //  Result.Time := StrToFloat(edDelay.Text);
end;

procedure TfrmSeq.UpdateFrameEdit;
begin
  edFrame.Text := IntToStr(CurFrame);
end;

procedure TfrmSeq.UpdateView;
begin
  Settings := fSeq[CurFrame];
  MainForm.DrawScene;
end;

procedure TfrmSeq.btnClearClick(Sender: TObject);
begin
  SetLength(fSeq, 0);
  CurFrame := 0;
  UpdateFrameEdit;
end;

procedure TfrmSeq.btnBeginClick(Sender: TObject);
begin
  CurFrame := 0;
  UpdateView;
  UpdateFrameEdit;
end;

procedure TfrmSeq.btnBackClick(Sender: TObject);
begin
  CurFrame := Max(0, CurFrame - 1);
  UpdateView;
  UpdateFrameEdit;
end;

procedure TfrmSeq.btnForwClick(Sender: TObject);
begin
  CurFrame := Min(Length(fSeq) - 1, CurFrame + 1);
  UpdateView;
  UpdateFrameEdit;
end;

procedure TfrmSeq.btnEndClick(Sender: TObject);
begin
  CurFrame := Max(0, Length(fSeq) - 1);
  UpdateView;
  UpdateFrameEdit;
end;

procedure TfrmSeq.btnSnapClick(Sender: TObject);
var
  i                 : integer;
begin
  SetLength(fSeq, Max(Length(fSeq), CurFrame + 1));
  for i := Length(fSeq) - 1 downto CurFrame + 1 do
    fSeq[i] := fSeq[i - 1];
  fSeq[CurFrame] := GrabSettings;
  Inc(CurFrame);
  UpdateFrameEdit;
end;

procedure TfrmSeq.btnNoSnapClick(Sender: TObject);
var
  i                 : integer;
begin
  for i := CurFrame to Length(fSeq) - 2 do
    fSeq[i] := fSeq[i + 1];
  SetLength(fSeq, Max(0, Length(fSeq) - 1));
  CurFrame := Min(CurFrame, Length(fSeq) - 1);
  UpdateView;
  UpdateFrameEdit;
end;

procedure TfrmSeq.btnPlayStopClick(Sender: TObject);
var
  i, i0, i9         : integer;
  f                 : integer;
  d                 : integer;
  PrevFrame         : TSettings;
  Save0             : TSettings;
  Scale0, ScaleD    : TMyFloat;
  xBase0, xBaseD    : TMyFloat;
  yBase0, yBaseD    : TMyFloat;
  RotXPos0, RotXPosD: TMyFloat;
  RotZPos0, RotZPosD: TMyFloat;
  ScaleBarPos0, ScaleBarPosD: TMyFloat;
  OffsetXPos0, OffsetXPosD: TMyFloat;
  OffsetYPos0, OffsetYPosD: TMyFloat;
begin
  PrevFrame := GrabSettings;
  Save0 := fSeq[0];
  d := (Sender as TToolBarButton97).Tag;
  if d < 0 then begin
    i9 := -1;
    i0 := Pred(Length(fSeq));
  end
  else begin
    i0 := 0;
    i9 := Length(fSeq);
  end;
  i := i0;
  while i <> i9 do begin
    if fSeq[i].Frames > 0 then begin
      Scale0 := PrevFrame.Scale;
      ScaleD := (fSeq[i].Scale - PrevFrame.Scale) / fSeq[i].Frames;
      xBase0 := PrevFrame.xBase;
      xBaseD := (fSeq[i].xBase - PrevFrame.xBase) / fSeq[i].Frames;
      yBase0 := PrevFrame.yBase;
      yBaseD := (fSeq[i].yBase - PrevFrame.yBase) / fSeq[i].Frames;
      RotXPos0 := PrevFrame.RotXPos;
      RotXPosD := (fSeq[i].RotXPos - PrevFrame.RotXPos) / fSeq[i].Frames;
      RotZPos0 := PrevFrame.RotZPos;
      RotZPosD := (fSeq[i].RotZPos - PrevFrame.RotZPos) / fSeq[i].Frames;
      ScaleBarPos0 := PrevFrame.ScaleBarPos;
      ScaleBarPosD := (fSeq[i].ScaleBarPos - PrevFrame.ScaleBarPos) / fSeq[i].Frames;
      OffsetXPos0 := PrevFrame.OffsetXPos;
      OffsetXPosD := (fSeq[i].OffsetXPos - PrevFrame.OffsetXPos) / fSeq[i].Frames;
      OffsetYPos0 := PrevFrame.OffsetYPos;
      OffsetYPosD := (fSeq[i].OffsetYPos - PrevFrame.OffsetYPos) / fSeq[i].Frames;
      f := fSeq[i].Frames;
      while f > 0 do begin
        with fSeq[0] do begin
          Scale := Scale0;
          xBase := xBase0;
          yBase := yBase0;
          RotXPos := Round(RotXPos0);
          RotZPos := Round(RotZPos0);
          //          ScaleBarPos := Round(ScaleBarPos0);
          //          OffsetXPos := Round(OffsetXPos0);
          //          OffsetYPos := Round(OffsetYPos0);
        end;
        CurFrame := 0;
        UpdateView;
        Application.ProcessMessages;
        if fSeq[i].Time > 0 then
          Sleep(Round(1000 * fSeq[i].Time))
        else
          Sleep(Round(1000 * StrToFloat(edDelay.Text)));
        Scale0 := Scale0 + ScaleD;
        xBase0 := xBase0 + xBaseD;
        yBase0 := yBase0 + yBaseD;
        RotXPos0 := RotXPos0 + RotXPosD;
        RotZPos0 := RotZPos0 + RotZPosD;
        ScaleBarPos0 := ScaleBarPos0 + ScaleBarPosD;
        OffsetXPos0 := OffsetXPos0 + OffsetXPosD;
        OffsetYPos0 := OffsetYPos0 + OffsetYPosD;
        Dec(f);
      end
    end
    else begin
      CurFrame := i;
      UpdateFrameEdit;
      UpdateView;
      Application.ProcessMessages;
      if fSeq[i].Time > 0 then
        Sleep(Round(1000 * fSeq[i].Time))
      else
        Sleep(Round(1000 * StrToFloat(edDelay.Text)));
    end;
    PrevFrame := fSeq[i];
    Inc(i, d);
  end;
  fSeq[0] := Save0;
end;

procedure TfrmSeq.btnOpenClick(Sender: TObject);
var
  f                 : TFileStream;
  i                 : integer;
begin
  if OpenDialog1.Execute then begin
    f := TFileStream.Create(OpenDialog1.FileName, fmOpenRead);
    try
      f.ReadBuffer(i, SizeOf(Integer));
      SetLength(fSeq, i);
      for i := 0 to Pred(Length(fSeq)) do
        f.ReadBuffer(fSeq[i], SizeOf(fSeq[i]));
    finally
      f.Free;
    end
  end;
  CurFrame := 0;
  UpdateView;
end;

procedure TfrmSeq.btnSaveClick(Sender: TObject);
var
  f                 : TFileStream;
  i                 : integer;
begin
  if SaveDialog1.Execute then begin
    f := TFileStream.Create(SaveDialog1.FileName, fmCreate);
    try
      i := Length(fSeq);
      f.WriteBuffer(i, SizeOf(Integer));
      for i := 0 to Pred(Length(fSeq)) do
        f.WriteBuffer(fSeq[i], SizeOf(fSeq[i]));
    finally
      f.Free;
    end
  end
end;

procedure TfrmSeq.Attach(Form: TForm);
begin
  SeqPanel.Parent := Form;
end;

procedure TfrmSeq.Detach;
begin
  SeqPanel.Parent := Self;
end;

function TfrmSeq.Attached(Form: TForm): boolean;
begin
  Result := SeqPanel.Parent = Form;
end;

end.
